/*==============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with "yanera 2.0"; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
==============================================================================*/ 
/*============================================================================*/
/** \file yanera_data.c
 *  \brief This code reads data files and resoltion files.

  This code reads data files and resoltion files.
 */
#define _CRT_SECURE_NO_DEPRECATE

#include "yanera.h"
#include "yanera_data.h"

/*=========================================================================*/
void yanera_load_data(yanera_container *yanera)
{
  unsigned int  i, j, k;
	char          buff[120];
	unsigned int  ndat;
	double        dtmp;
	FILE         *fp = NULL;

  for (k=0; k<yanera->number_of_models; k++)
  {
    /*--------------------------------------------
     * Open the data file
     */
    fp = fopen(yanera->data[k].file_name, "r");
    if (fp==NULL)
    { 
      yanera_error("Cannot open data file.", __FILE__, __LINE__, yanera);
    }
    
    /*--------------------------------------------
     *  Count the number of data points
     */	
    ndat= 0;
    memset(buff, '\0', 120);
    while(fgets(buff, 119, fp) != NULL)
    {
    	j = sscanf(buff, "%lf %lf %lf", &dtmp, &dtmp, &dtmp);
    	if ((j == 2) || (j == 3)) ndat++;
    	memset(buff, '\0', 120);
    }
    rewind(fp);

    /*--------------------------------------------
     *  Allocate memory for the data
     */	
    yanera->data[k].n = ndat;
    yanera->data[k].q = (double *)calloc(yanera->data[k].n,sizeof(double));
    yanera->data[k].R = (double *)calloc(yanera->data[k].n,sizeof(double));
    yanera->data[k].e = (double *)calloc(yanera->data[k].n,sizeof(double));
	
    /*--------------------------------------------
     *  Read in  the data
     */	
    ndat= 0;
    memset(buff, '\0', 120);
    while(fgets(buff, 119, fp) != NULL)
    {
    	j = sscanf(buff, "%lf %lf %lf", &yanera->data[k].q[ndat], \
	                                    &yanera->data[k].R[ndat], \
								                      &yanera->data[k].e[ndat]);
	    /* 
	     *  If we want error in R from file, but don't get it,
	     *  what should we do? For now, set e=1.
    	 */
	    if ((j == 2) && (yanera->misc.fit_weighting == WEIGHT_DATA)) 
	    { yanera->data[k].e[ndat] = 1.0; }
	    if ((j == 2) || (j == 3)) ndat++;
	    memset(buff, '\0', 120);
    }
    fclose(fp);
	
    /*--------------------------------------------
     *  Set the error in R for other cases
     */	
    for (i=0; i<yanera->data[k].n; i++)
    {
	    if      (yanera->misc.fit_weighting == WEIGHT_NONE) 
	    { yanera->data[k].e[i] = 1.0; }
	    else if (yanera->misc.fit_weighting == WEIGHT_R)    
	    { yanera->data[k].e[i] = yanera->data[k].R[i]; }
	    else if (yanera->misc.fit_weighting == WEIGHT_SQRT) 
	    { yanera->data[k].e[i] = sqrt(fabs(yanera->data[k].R[i])); }
    }
  }
  
  return;
}
/*============================================================================*/
void yanera_read_resolution(yanera_container *yanera) 
{
  unsigned int     j, k;
	char             buff[120];
	unsigned int     ndat;
	double           dtmp;
	FILE            *fp = NULL;

  for (k=0; k<yanera->number_of_models; k++)
  {
    if ((yanera->data[k].resolution.type == RESOLUTION_ARRAY_CONSTANT) ||
        (yanera->data[k].resolution.type == RESOLUTION_ARRAY_RELATIVE))
      {
    
      if (yanera->data[k].resolution.n > 0) 
      { 
        yanera_error("Resolution cannot be both array and value.", \
                      __FILE__, __LINE__, yanera);
      }
     /*--------------------------------------------
       * Open the data file
       */
      fp = fopen(yanera->data[k].resolution.file_name, "r");
      if (fp==NULL)
      { 
        yanera_error("Cannot open resultion array file.", \
                     __FILE__, __LINE__, yanera);
      }

      /*--------------------------------------------
       *  Count the number of data points
       */	
      ndat= 0;
      memset(buff, '\0', 120);
      while(fgets(buff, 119, fp) != NULL)
      {
	      j = sscanf(buff, "%lf %lf", &dtmp, &dtmp);
	      if (j == 2) ndat++;
	      memset(buff, '\0', 120);
      }
      rewind(fp);

      /*--------------------------------------------
       *  Allocate memory for the data
       */	
      yanera->data[k].resolution.n     = ndat;
      yanera->data[k].resolution.value = \
        (double *)calloc(yanera->data[k].resolution.n, sizeof(double));
      yanera->data[k].resolution.q     = \
        (double *)calloc(yanera->data[k].resolution.n, sizeof(double));
	
      /*--------------------------------------------
       *  Read in  the data
       */	
      ndat= 0;
      memset(buff, '\0', 120);
      while(fgets(buff, 119, fp) != NULL)
      {
	      j = sscanf(buff, "%lf %lf", &yanera->data[k].resolution.q[ndat], \
	                  &yanera->data[k].resolution.value[ndat]);
	      if (j == 2) ndat++;            
	      memset(buff, '\0', 120);
      }
      fclose(fp);
    }
    
    if (yanera->data[k].resolution.type != RESOLUTION_NONE)
    {
      if (yanera->data[k].resolution.n == 0) 
      { 
        yanera_error("Missing resolution value.", \
                     __FILE__, __LINE__, yanera);
      }
    }
    
  }
  
  return;
}
