/*============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with "yanera 2.0"; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
==============================================================================*/
/*============================================================================*/
/** \file yanera_model_build.h
	 \brief This file is used to build the complete models for evaluation

The idea behind the XML file is that additional models beyond the first need only specify where they differ to the first.

Therefore, after reading the XML file, only the first model is specified completely. Any additional models need to be filled in completely before that can be evaluated. 

This file will copy the base model the correct number of times, and then apply modifications specified in the XML representation.
 */
/*============================================================================*/
/** \brief Fills in all models with the base model. 

\param yanera The \c yanera_container to fill in the complete models.

Fills in all models with the base model, then modifies them according to the XML file. 

First step is to modify the base model by the extra models. Second step is to connect the extra model to the common components, if any. The last step is to convert any function expressions form infix to postfix.
 */
void yanera_complete_models(yanera_container *yanera);
/*============================================================================*/
/** \brief Makes a copy of a \p yanera_model

\param destination A newly allocated \p yanera_model.
\param source A newly allocated \p yanera_model.

This function does not copy the slabs of the model. It is not likely this function will be called after slabs are created.
 */
void copyModel(yanera_model *destination, yanera_model *source);
/*============================================================================*/
/** \brief Alters models to reflect differences from first, base model.

\param destination The \c yanera_model that will be modified.
\param source The \c yanera_model that has the differences.


Alters models to reflect differences from first, base model.
 */
void modifyModel(yanera_model *destination, yanera_model *source);
/*============================================================================*/
/** \brief Connects common components with their layers.

\param destination The \c yanera_model that will be modified.
\param component The \c yanera_layer specifying component parts.


Connects common components with their layers.
 */
void modifyComponents(yanera_model *destination, yanera_layer *component);
