/*==============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with "yanera 2.0"; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
==============================================================================*/ 
/*============================================================================*/
/** \file yanera_postfix.h
	  \brief This file has the functions needed to convert functions into postfix expressions.

In order to evaluate a user supplied function expression of a layer (eg. a function like: a*exp(-(z-12.2)/b), where a,b are adjustable varaibles.), we have to convert the expression to from infix to postfix notation. This file has all the required functions for this.

Operator Precendence: (5 = evaluated first)
- 5. () and functions
- 4. unary plus/minus
- 3. power ^
- 2. *,/
- 1. +,-
.
*/
/** --------------------------------------------------------------------------- 
 *  \brief Operator types
 *
 *  The purpose of enumerating the operator types is to set the syntax 
 *   and grammer of infix notation.
 */
enum operator_type{
	NONE,
	NUMBER,
	ORDINATE,
	PARAMETER,
	OPERATOR_MULTADD,
	OPERATOR_POWER,
	OPERATOR_UNARY,
	OPERATOR_FUNCTION,
	PARENTHESIS_OPEN,
	PARENTHESIS_CLOSE
};
/*==========================================================================*/
/**
 * Parsing a floating point number requires character-by-character placing
 * of the number into a string, followed by conversion.
 */
char         *number_string;
/*==========================================================================*/
/**
 * The evaluation stack is for the evaluation of the postfix_queue.
 * 
 * The stack list and data structure is defined here. 
 */
struct evaluation_stack_element {
	double                           d;    /**< FILL IN */
	struct evaluation_stack_element *next; /**< FILL IN */
};
/**
 * The evaluation stack is for the evaluation of the postfix_queue.
 * 
 * The top of the stack list is defined here. 
 */
struct evaluation_stack {
	unsigned short                   n;   /**< FILL IN */
	struct evaluation_stack_element *top; /**< FILL IN */
};
/**
 * The evaluation stack is for the evaluation of the postfix_queue.
 * 
 * The external storage data structure is defined here. 
 */
struct evaluation_stack evaluation_stack;
/*==========================================================================*/
/**
 * The operator stack is used to hold operators for sorting be precendence
 * during prefix evaluation.
 * 
 * The stack list and storage data structure is defined here. 
 */
struct operator_stack_element {
	short                   prec;           /**< Operator precednece */
	enum operator_type             type;    /**< Operator type */
	char                           oper[4]; /**< Operator text */
	struct operator_stack_element *next;    /**< Linked list */
};
/**
 * The operator stack is used to hold operators for sorting be precendence
 * during prefix evaluation.
 * 
 * The top of the stack list is defined here. 
 */
struct operator_stack {
	unsigned short                 n;    /**< Number of elements in stack */
	struct operator_stack_element *top;	 /**< Top of stack */
};
/**
 * A global declaration of the operator stack.
 */
struct operator_stack operator_stack;
/*==============================================================================
  MISC
==============================================================================*/
short parmindex(char c, char *parm_name);
void parse_expression(yanera_postfix *pq, \
                      char *expression, char *parm_name, short *parm);
void parameter_decision(yanera_postfix *pq, const char c[4], short indx);
/*==============================================================================
  NUMBER_STRING
==============================================================================*/
void append_number_string(char c);
void stop_number_string(yanera_postfix *pq);
/*==============================================================================
 POSTFIX_QUEUE
==============================================================================*/
yanera_postfix *initizlize_postfix_queue(void);
void enqueue_postfix_queue(yanera_postfix *pq, \
                           const char *c, double d, short indx, short type);
void print_postfix(yanera_postfix *pq, double x, double *params);
double evaluate_postfix(yanera_postfix *pq, double z, double *params);
void free_postfix_queue(yanera_postfix *pq);
/*==============================================================================
 OPERATOR_STACK
==============================================================================*/
void initizlize_operator_stack(void);
struct operator_stack_element *operator_stack_element_new( \
			                         const char *oper, short prec,short type);
void push_operator_stack(struct operator_stack_element *o);
struct operator_stack_element *pop_operator_stack(void);
short operator_stack_is_empty(void);
short operator_stack_top_precedence(void);
void operator_decision(yanera_postfix *pq, \
                       const char oper[4], short prec, short type);
/*==============================================================================
  EVALUATION_STACK
==============================================================================*/
void initizlize_evaluation_stack(void);
void push_evaluation_stack(double d);
double pop_evaluation_stack(void);


